#pragma once
#include "B4RDefines.h"
#include "RCSwitch.h"

    //~Event:  DataAvailable()
    //~Version: 1.0 
    //~Author - 
    //~Libray from https://github.com/sui77/rc-switch/wiki

namespace B4R {
    //~shortname:RCSwitch
    /*     
    * Operate 433/315 MHz radio controlled devices. This will most likely work with all popular low cost power outlet sockets.
	* If yours doesn't work, you might need to adjust the pulse length.
	* Supported chipsets: SC5262 - SC5272 - HX2262 - HX2272 - PT2262 - PT2272 - EV1527 - RT1527 - FP1527 - HS1527
	* Intertechno outlets HT6P20X
    */
    class B4RRCSwitch
    {
        private:
            uint8_t backend[sizeof(RCSwitch)];
            RCSwitch* rcs;
            PollerNode pnode;
            SubVoidVoid DataAvailableSub;
            static void looper(void* b);
            RCSwitch::Protocol protocol; 

        public: 
    //Initializes the library.
            void Initialize();
    //Enable a receiver connected to Receiverpin (only pin with interrupt)        
            void enableReceive(SubVoidVoid DataAvailableSub, int16_t interrupt);
            
            void switchOn(int16_t nGroupNumber, int16_t nSwitchNumber);
            void switchOff(int16_t nGroupNumber, int16_t nSwitchNumber);
            void switchOn1(B4RString* sGroup, int16_t nSwitchNumber);
            void switchOff1(B4RString* sGroup, int16_t nSwitchNumber);
            void switchOn2(byte sFamily, int16_t nGroup, int16_t nDevice);
            void switchOff2(byte sFamily, int16_t nGroup, int16_t nDevice);
            void switchOn3(B4RString* sGroup, B4RString* sDevice);
            void switchOff3(B4RString* sGroup, B4RString* sDevice);
            void switchOn4(byte sGroup, int16_t nDevice);
            void switchOff4(byte sGroup, int16_t nDevice);
            void sendTriState(B4RString* sCodeWord);
    // Sends a code but only length part        
            void send(uint32_t code, uint16_t length);
    // Sends a code in string format: a binary code word consisting of the letter 0, 1        
            void send1(B4RString* sCodeWord);
    //  Disable receiving data        
            void disableReceive();
    // After reading the received data, call ResetAvailable       
            void resetAvailable();
    // Returns the received code         
            uint32_t getReceivedValue();
    // Returns the received bit length        
            uint16_t getReceivedBitlength();
    // Returns the received delay         
            uint16_t getReceivedDelay();
    // Returns the received protocol        
            uint16_t getReceivedProtocol();
    // Returns the received data in raw format              
            ArrayUInt* getReceivedRawdata();
    // Enable a transmitter connected to Transmitterpin         
            void enableTransmit(int16_t nTransmitterPin);
    // Disable transmissions            
            void disableTransmit();
    // Sets the pulselength        
            void setPulseLength(int16_t nPulseLength);
    // Sets the number of times the code will be sent        
            void setRepeatTransmit(int16_t nRepeatTransmit);
    // Set Receiving Tolerance       
            void setReceiveTolerance(int16_t nPercent);
    // Sets the protocol to transmit            
            void setProtocol(uint16_t pulseLength, uint8_t syncFactor_high, uint8_t syncFactor_low, uint8_t zero_high, uint8_t zero_low, uint8_t one_high, int8_t one_low, bool invertedSignal );
    // Sets the protocol to transmit by internal protocol number
            void setProtocol1(int16_t nProtocol);                                //#Meth
    // Sets the protocol number to transmit +  pulselength           
            void setProtocol2(int16_t nProtocol, int16_t nPulseLength);       //#Meth
            

	//rcs.RepeatTransmit = rcs.Repeat_1_time
			#define /*Byte Repeat_1_time;*/   B4R_Repeat_1_time	            4
	//rcs.RepeatTransmit = rcs.Repeat_2_times
			#define /*Byte Repeat_2_times;*/   B4R_Repeat_2_times               6
	//rcs.RepeatTransmit = rcs.Repeat_3_times
			#define /*Byte Repeat_3_times;*/   B4R_Repeat_3_times               8
	//rcs.RepeatTransmit = rcs.Repeat_4_times
	    	#define /*Byte Repeat_4_times;*/   B4R_Repeat_4_times                       12
    //rcs.RepeatTransmit = rcs.Repeat_5_times
			#define /*Byte Repeat_5_times;*/   B4R_Repeat_5_times               14
	//rcs.RepeatTransmit = rcs.Repeat_6_times
			#define /*Byte Repeat_6_times;*/   B4R_Repeat_6_times               18
	//rcs.RepeatTransmit = rcs.Repeat_7_times                          
			#define /*Byte Repeat_7_times;*/   B4R_Repeat_7_times               20
	//rcs.RepeatTransmit = rcs.Repeat_8_times
	       	#define /*Byte Repeat_8_times;*/   B4R_Repeat_8_times                       25
	//rcs.RepeatTransmit = rcs.Repeat_9_times
			#define /*Byte Repeat_9_times;*/   B4R_Repeat_9_times               26
	//rcs.RepeatTransmit = rcs.Repeat_10_times
			#define /*Byte Repeat_10_times;*/   B4R_Repeat_10_times             30
    };
}
